library slang_retail_assistant;

import 'dart:collection';
import 'dart:core';
import 'package:flutter/services.dart';

part 'src/assistant_ui_position.dart';

part 'src/assistant_config.dart';

part 'src/search_user_journey.dart';

part 'src/assistant_ui.dart';

part 'src/search_info.dart';

class SlangRetailAssistant {
  SlangRetailAssistant._();

  static const MethodChannel _channel =
      const MethodChannel('plugins.flutter.io/slang_retail_assistant');

  static late RetailAssistantAction _action;
  static late RetailAssistantLifeCycleObserver _lifeCycleObserver;
  static late SearchUserJourney _searchUserJourney;

  static Future<void> _methodCallHandler(MethodCall call) async {
    switch (call.method) {
      case 'onSearch':
        _searchUserJourney = new SearchUserJourney._();
        SearchAppState searchAppState = _action.onSearch(
            new SearchInfo._(call.arguments), _searchUserJourney);
        if (searchAppState != SearchAppState.WAITING) {
          _searchUserJourney.notifyAppState(searchAppState);
        }
        break;
      case 'onAssistantError':
        final Map<dynamic, dynamic> _data = call.arguments;
        Map<String, String> arguments = new HashMap();
        arguments["description"] = _data["description"];
        _action.onAssistantError(arguments);
        break;
      case 'onAssistantInitSuccess':
        _lifeCycleObserver.onAssistantInitSuccess();
        break;
      case 'onAssistantInitFailure':
        final Map<dynamic, dynamic> _data = call.arguments;
        String description = _data["type"];
        _lifeCycleObserver.onAssistantInitFailure(description);
        break;
      case 'onAssistantInvoked':
        _lifeCycleObserver.onAssistantInvoked();
        break;
      case 'onAssistantClosed':
        final Map<dynamic, dynamic> _data = call.arguments;
        bool isCancelled = _data["isCancelled"];
        _lifeCycleObserver.onAssistantClosed(isCancelled);
        break;
      case 'onAssistantLocaleChanged':
        final Map<dynamic, dynamic> _data = call.arguments;
        Map<String, String> locale = new HashMap();
        locale["country"] = _data["country"];
        locale["language"] = _data["language"];
        _lifeCycleObserver.onAssistantLocaleChanged(locale);
        break;
      case 'onUnrecognisedUtterance':
        final Map<dynamic, dynamic> _data = call.arguments;
        String utterance = _data["utterance"];
        _lifeCycleObserver.onUnrecognisedUtterance(utterance);
        break;
      case 'onUtteranceDetected':
        final Map<dynamic, dynamic> _data = call.arguments;
        String utterance = _data["utterance"];
        _lifeCycleObserver.onUtteranceDetected(utterance);
        break;
      case 'onOnboardingSuccess':
        _lifeCycleObserver.onOnboardingSuccess();
        break;
      case 'onOnboardingFailure':
        _lifeCycleObserver.onOnboardingFailure();
        break;
      case 'onOnboardingFailure':
        _lifeCycleObserver.onOnboardingFailure();
        break;
      case 'onMicPermissionDenied':
        _lifeCycleObserver.onMicPermissionDenied();
        break;
      default:
        print(
            'SlangRetailAssistant: Ignoring invoke from native. This normally shouldn\'t happen.');
    }
  }

  static void initialize(AssistantConfiguration assistantConfiguration) {
    _channel.invokeMapMethod(
        "initialize", assistantConfiguration._covertToHashmap());
    _channel.setMethodCallHandler(_methodCallHandler);
  }

  static void setAction(RetailAssistantAction action) {
    _channel.invokeMapMethod("setAction");
    _action = action;
  }

  static void setLifecycleObserver(
      RetailAssistantLifeCycleObserver lifeCycleObserver) {
    _channel.invokeMapMethod("setLifecycleObserver");
    _lifeCycleObserver = lifeCycleObserver;
  }

  static void startConversation(AssistantUserJourney assistantUserJourney) {
    _channel.invokeMapMethod(
        "startConversation", {"userJourney": assistantUserJourney.stringValue});
  }

  static void notifyNonVoiceUserJourney(
      AssistantUserJourney assistantUserJourney,
      Map<String, String> journeyInfo) {
    _channel.invokeMapMethod(
        "notifyNonVoiceUserJourney", {
          "userJourney": assistantUserJourney.stringValue,
          "userJourneyInfo" : journeyInfo
    });
  }

  static void notifyNonVoiceSearch(String searchTerm) {
    notifyNonVoiceUserJourney(AssistantUserJourney.SEARCH,
        {"searchString": searchTerm});
  }

  static _AssistantUI getUI() {
    return _AssistantUI(_channel);
  }
}

class RetailAssistantAction {
  SearchAppState onSearch(
      SearchInfo searchInfo, SearchUserJourney searchUserJourney) {
    return SearchAppState.UNSUPPORTED;
  }

  void onAssistantError(Map<String, String> assistantError) {}
}

class RetailAssistantLifeCycleObserver {
  void onAssistantInitSuccess() {}

  void onAssistantInitFailure(String description) {}

  void onAssistantInvoked() {}

  void onAssistantClosed(bool isCancelled) {}

  void onAssistantLocaleChanged(Map<String, String> locale) {}

  void onUnrecognisedUtterance(String utterance) {}

  void onUtteranceDetected(String utterance) {}

  void onOnboardingSuccess() {}

  void onOnboardingFailure() {}

  void onMicPermissionDenied() {}
}
